package com.jay.monitor.data.client;

import com.alipay.remoting.InvokeContext;
import com.alipay.remoting.rpc.RpcClient;
import com.jay.monitor.data.core.enums.MonitorDataConstants;
import com.jay.monitor.data.core.model.serializable.base.BaseDTO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Queue;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 监控数据客户端传输列
 *
 * @author ： liukx
 * @time ： 2020/8/26 - 14:25
 */
public class MonitorDataClient implements Runnable {

    private Logger logger = LoggerFactory.getLogger(getClass());

    private String host;

    private Integer port = MonitorDataConstants.DEFAULT_PORT;

    private String addr;

    private RpcClient client = new RpcClient();

    private ExecutorService executorService = Executors.newSingleThreadExecutor();

    private Integer queueSize = 1000;

    private Queue<BaseDTO> queueData;

    /**
     * 全局上下文传递信息，用于存储一些公共信息
     */
    private InvokeContext invokeContext = new InvokeContext();

    public MonitorDataClient(String host, Integer port) {
        this.host = host;
        this.port = port;
        this.addr = host + ":" + port;

    }

    /**
     * 初始化数据
     */
    public void init() {
        queueData = new ArrayBlockingQueue<>(getQueueSize());
        executorService.execute(this);
        client.startup();
    }

    public Integer getQueueSize() {
        return queueSize;
    }

    public void setQueueSize(Integer queueSize) {
        this.queueSize = queueSize;
    }

    public String getAddr() {
        return addr;
    }

    /**
     * 单向发送
     *
     * @param data
     * @throws Exception
     */
    private void sendOnWay(Object data) throws Exception {
        client.oneway(getAddr(), data, invokeContext);
    }

    /**
     * 发送数据
     *
     * @param data
     */
    public void sendObjectMsg(BaseDTO data) {
        queueData.add(data);
    }

    public void addInvokeContext(String key, String value) {
        invokeContext.put(key, value);
    }

    @Override
    public void run() {
        logger.info(" 开启监控数据传输线程 ... ");
        while (true) {
            BaseDTO baseDTO = queueData.poll();
            try {
                if (baseDTO != null) {
                    sendOnWay(baseDTO);
                }
            } catch (Exception e) {
                logger.error("monitor consumer fail", e);
            } finally {
                try {
                    Thread.sleep(5);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                    logger.error("数据传输线程失败", e);
                }
            }

        }
    }
}
